"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var PropTypes = require("prop-types");
var layout_component_1 = require("../layout.component");
var layout_constants_1 = require("../layout.constants");
var react_1 = require("@storybook/react");
var string_1 = require("@devexperts/utils/dist/string/string");
var layout_utils_1 = require("../layout.utils");
var dnd_context_1 = require("../dnd-context");
var css = require("./layout.page.styl");
var layout_redux_1 = require("../layout.redux");
var Button_1 = require("../../Button/Button");
var Demo_1 = require("../../demo/Demo");
var layoutCss = require("./layout.styl");
var scrollableCss = require("./scrollable.styl");
var withTheme_1 = require("../../../utils/withTheme");
var theme = {
    container: css.container,
};
var layoutTheme = withTheme_1.mergeThemes({
    item: css.layout__item,
    placeholder: css.layout__placeholder,
    placeholder_fits: css.layout__placeholder_fits,
    placeholder_maximized: css.layout__placeholder_maximized,
    resizeHandler: css.resizeHandler,
}, layoutCss, {
    Scrollable: scrollableCss,
});
var prevent = function (e) { return e.preventDefault(); };
var demoPlaceholder = function (props) {
    var className = props.className, fits = props.fits, isMaximized = props.isMaximized;
    return (React.createElement("div", { className: className },
        !fits && 'X',
        isMaximized && (React.createElement("div", { className: css.placeholder__hint },
            "Chart widget is in maximized mode.",
            React.createElement("small", null, "Exit maximized mode to add new widgets to workspace")))));
};
var DemoItem = /** @class */ (function (_super) {
    tslib_1.__extends(DemoItem, _super);
    function DemoItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DemoItem.prototype.render = function () {
        //eslint-disable-next-line
        var _a = this.props, color = _a.color, id = _a.id, connectDragSource = _a.connectDragSource, onRemove = _a.onRemove;
        var _b = this.context, handleMaximizeClick = _b.handleMaximizeClick, checkIsWidgetMaximized = _b.checkIsWidgetMaximized; // TODO: context must be here, but typescript complains! research!
        var isMaximized = checkIsWidgetMaximized(id);
        var style = {
            backgroundColor: color,
        };
        var removeTheme = {
            container: css.item__remove,
        };
        var actions = (React.createElement("div", { className: css.item__actions },
            React.createElement(Button_1.Button, { onMouseDown: prevent, onClick: handleMaximizeClick(id), theme: removeTheme }, "M"),
            !isMaximized && (React.createElement(Button_1.Button, { onMouseDown: prevent, onClick: onRemove, theme: removeTheme }, "X"))));
        return (React.createElement("div", { style: style, className: css.item },
            connectDragSource &&
                connectDragSource(React.createElement("div", { className: css.item__dragHandler },
                    React.createElement("span", null, "Drag Me"),
                    actions), {
                    dropEffect: 'none',
                }),
            !connectDragSource && (React.createElement("div", { className: css.item__dragHandler },
                React.createElement("span", null, isMaximized ? "I'm maximized" : 'Try to Drag Me'),
                actions))));
    };
    DemoItem.contextTypes = {
        handleMaximizeClick: PropTypes.func,
        checkIsWidgetMaximized: PropTypes.func,
    };
    return DemoItem;
}(React.Component));
var minWidth = 500; //px
var minHeight = 125; //px
var layout = {
    id: 'root',
    size: 1,
    orientation: layout_constants_1.LayoutOrientation.Horizontal,
    items: [
        {
            id: 'left',
            props: {
                color: 'lightgrey',
                minWidth: minWidth,
                minHeight: minHeight,
            },
            size: 0.5,
        },
        {
            id: 'right',
            size: 0.5,
            orientation: layout_constants_1.LayoutOrientation.Vertical,
            items: [
                {
                    id: string_1.uuid(),
                    props: {
                        color: 'yellow',
                        minWidth: minWidth,
                        minHeight: minHeight,
                    },
                    size: 0.25,
                },
                {
                    id: string_1.uuid(),
                    props: {
                        color: 'blue',
                        minWidth: minWidth,
                        minHeight: minHeight,
                    },
                    size: 0.25,
                },
            ],
        },
    ],
};
var value = layout_utils_1.prepareLayout(layout);
//make only top level layout a dnd context
var DNDLayout = dnd_context_1.DNDContext(layout_component_1.Layout);
var LayoutStory = /** @class */ (function (_super) {
    tslib_1.__extends(LayoutStory, _super);
    function LayoutStory() {
        //noinspection JSDuplicatedDeclaration
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            value: value,
            side: 'Right',
            isFixed: false,
            color: 'darkred',
        };
        _this.onHandleItem = function (e) {
            _this.setState({
                root: e.target.value,
            });
        };
        _this.onHandleSide = function (e) {
            _this.setState({
                side: e.target.value,
            });
        };
        _this.onHandleInputColor = function (e) {
            _this.setState({
                color: e.target.value,
            });
        };
        _this.insertItem = function () {
            var _a = _this.state, value = _a.value, root = _a.root, side = _a.side, color = _a.color;
            var newSide = layout_constants_1.ItemSide.Top;
            switch (side) {
                case 'Top':
                    newSide = layout_constants_1.ItemSide.Top;
                    break;
                case 'Bottom':
                    newSide = layout_constants_1.ItemSide.Bottom;
                    break;
                case 'Left':
                    newSide = layout_constants_1.ItemSide.Left;
                    break;
                case 'Right':
                    newSide = layout_constants_1.ItemSide.Right;
                    break;
            }
            if (root) {
                var newValue = layout_redux_1.reducer(value, layout_redux_1.insertNativeAt({
                    color: color,
                    minWidth: minWidth,
                    minHeight: minHeight,
                }, root, newSide));
                console.log(newValue);
                _this.setState({
                    value: newValue,
                });
            }
        };
        _this.handleClearLayout = function () {
            _this.setState({
                maximizedId: void 0,
                value: layout_utils_1.prepareLayout(tslib_1.__assign(tslib_1.__assign({}, layout), { items: [] })),
            });
        };
        _this.handleMaximizeClick = function (id) { return function (e) {
            var maximizedId = _this.state.maximizedId;
            _this.setState({
                maximizedId: maximizedId ? void 0 : id,
            });
        }; };
        _this.onChange = function (value) {
            var items = Object.keys(value.items).reduce(function (acc, id) {
                var _a;
                var item = value.items[id];
                if (item.type === layout_constants_1.LayoutModelItemType.Plain) {
                    item = tslib_1.__assign(tslib_1.__assign({}, item), { props: tslib_1.__assign({}, item.props) });
                }
                return tslib_1.__assign(tslib_1.__assign({}, acc), (_a = {}, _a[id] = item, _a));
            }, {});
            _this.setState({
                value: tslib_1.__assign(tslib_1.__assign({}, value), { items: items }),
            });
        };
        _this.checkIsWidgetMaximized = function (id) {
            return id === _this.state.maximizedId;
        };
        _this.onCheckboxChange = function () {
            _this.setState({
                isFixed: !_this.state.isFixed,
            });
        };
        _this.onDragStart = function (color) { return function (e) {
            layout_utils_1.setDragData(e.nativeEvent, {
                color: color,
                minWidth: minWidth,
                minHeight: minHeight,
            });
        }; };
        return _this;
    }
    LayoutStory.prototype.getChildContext = function () {
        return {
            handleMaximizeClick: this.handleMaximizeClick,
            checkIsWidgetMaximized: this.checkIsWidgetMaximized,
        };
    };
    LayoutStory.prototype.render = function () {
        var value = this.state.value;
        var items = Object.keys(value.items);
        items = items.filter(function (item) {
            return item !== value.root_id;
        });
        return (React.createElement(Demo_1.Demo, { theme: theme },
            React.createElement("div", null,
                React.createElement("div", { className: css.toolbar },
                    React.createElement("div", { className: css.toolbarControl },
                        React.createElement("label", { className: css.label },
                            React.createElement("input", { className: css.checkbox, type: "checkbox", onChange: this.onCheckboxChange }),
                            React.createElement("span", null, "isFixed"))),
                    React.createElement("div", { className: css.toolbarControl },
                        React.createElement(Button_1.Button, { onClick: this.handleClearLayout, isPrimary: true }, "Clear layout")),
                    React.createElement("div", { className: css.toolbarAddItem },
                        React.createElement("label", null, "Items"),
                        React.createElement("select", { onChange: this.onHandleItem, value: this.state.root },
                            React.createElement("option", { key: 0, value: "" }),
                            items.map(function (item, index) {
                                return (React.createElement("option", { key: index + 1, value: item }, item));
                            })),
                        React.createElement("label", null, "Side"),
                        React.createElement("select", { onChange: this.onHandleSide, value: this.state.side },
                            React.createElement("option", { value: "Top" }, "Top"),
                            React.createElement("option", { value: "Bottom" }, "Bottom"),
                            React.createElement("option", { value: "Left" }, "Left"),
                            React.createElement("option", { value: "Right" }, "Right")),
                        React.createElement("label", null, "Color"),
                        React.createElement("input", { type: "text", onChange: this.onHandleInputColor }),
                        React.createElement(Button_1.Button, { onClick: this.insertItem }, "Add Item"))),
                React.createElement("ul", { className: css.draggableItems },
                    React.createElement("li", { draggable: true, className: css.draggableItem, onDragStart: this.onDragStart('green') }, "green"),
                    React.createElement("li", { draggable: true, onDragStart: this.onDragStart('magenta'), className: css.draggableItem }, "magenta"),
                    React.createElement("li", { draggable: true, onDragStart: this.onDragStart('yellow'), className: css.draggableItem }, "yellow"),
                    React.createElement("li", { draggable: true, onDragStart: this.onDragStart('blue'), className: css.draggableItem }, "blue"),
                    React.createElement("li", { draggable: true, onDragStart: this.onDragStart('cyan'), className: css.draggableItem }, "cyan")),
                React.createElement("section", { className: css.layoutContainer },
                    React.createElement(DNDLayout, { theme: layoutTheme, ItemComponent: DemoItem, Placeholder: demoPlaceholder, debounceResize: 70, isFixed: this.state.isFixed, maximizedId: this.state.maximizedId, value: this.state.value, onChange: this.onChange, itemPropsFactory: function (data) { return (tslib_1.__assign(tslib_1.__assign({}, data), { minWidth: minWidth,
                            minHeight: minHeight })); }, minItemWidth: minWidth, minItemHeight: minHeight, resizeHandlerSize: 10 })))));
    };
    LayoutStory.childContextTypes = {
        handleMaximizeClick: PropTypes.func,
        checkIsWidgetMaximized: PropTypes.func,
    };
    return LayoutStory;
}(React.Component));
react_1.storiesOf('Layout', module).add('default', function () { return React.createElement(LayoutStory, null); });
//# sourceMappingURL=layout.page.js.map