import { TLayoutModelItem, ItemSide, TLayoutStructure, TMinimumLayoutModelItemProps } from './layout.constants';
declare enum ActionType {
    ItemRemove = 0,
    ItemInsertAt = 1,
    ItemInsertNativeAt = 2,
    ItemInsertNativeAtEmptyRoot = 3,
    ItemMoveTo = 4,
    ItemResize = 5
}
declare type TItemRemoveAction = {
    type: ActionType.ItemRemove;
    payload: {
        id: string;
    };
};
declare type TItemInsertAtAction<P extends TMinimumLayoutModelItemProps> = {
    type: ActionType.ItemInsertAt;
    payload: {
        id: string;
        side: ItemSide;
        item: TLayoutModelItem<P>;
    };
};
declare type TItemInsertNativeAtAction<P extends TMinimumLayoutModelItemProps> = {
    type: ActionType.ItemInsertNativeAt;
    payload: {
        id: string;
        side: ItemSide;
        props: P;
    };
};
declare type TItemInsertNativeAtEmptyRootAction<P extends TMinimumLayoutModelItemProps> = {
    type: ActionType.ItemInsertNativeAtEmptyRoot;
    payload: {
        props: P;
    };
};
declare type TItemMoveToAction = {
    type: ActionType.ItemMoveTo;
    payload: {
        id: string;
        side: ItemSide;
        destinationId: string;
    };
};
declare type TItemResizeAction = {
    type: ActionType.ItemResize;
    payload: {
        id1: string;
        size1: number;
        id2: string;
        size2: number;
    };
};
export declare type TLayoutAction<P extends TMinimumLayoutModelItemProps> = TItemRemoveAction | TItemInsertAtAction<P> | TItemMoveToAction | TItemResizeAction | TItemInsertNativeAtAction<P> | TItemInsertNativeAtEmptyRootAction<P>;
export declare const remove: (id: string) => TItemRemoveAction;
export declare const insertAt: <P extends TMinimumLayoutModelItemProps>(item: TLayoutModelItem<P>, id: string, side: ItemSide) => TItemInsertAtAction<P>;
export declare const insertNativeAt: <P extends TMinimumLayoutModelItemProps>(props: P, id: string, side: ItemSide) => TItemInsertNativeAtAction<P>;
export declare const insertNativeAtEmptyRoot: <P extends TMinimumLayoutModelItemProps>(props: P) => TItemInsertNativeAtEmptyRootAction<P>;
export declare const moveTo: (id: string, destinationId: string, side: ItemSide) => TItemMoveToAction;
export declare const resize: (id1: string, size1: number, id2: string, size2: number) => TItemResizeAction;
export declare const reducer: <P extends TMinimumLayoutModelItemProps>(state: TLayoutStructure<P> | undefined, action: TLayoutAction<P>) => TLayoutStructure<P>;
export {};
