"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var layout_constants_1 = require("./layout.constants");
var string_1 = require("@devexperts/utils/dist/string/string");
var getSum = function (items) {
    if (items.length === 0) {
        return 0;
    }
    return items.reduce(function (acc, item) { return acc + item; });
};
function fixFloatSizes(sizes, sum, precision) {
    //increase the last size if average value is not integer
    //for example for 3 items average size is 33.33...% and the last will be 33.34...%
    //33.33...% + 33.33...% + 33.34...% === 100%
    var sizesSum = Number(getSum(sizes).toFixed(precision));
    if (sizesSum > sum) {
        return sizes.map(function (size) { return Number(size.toFixed(precision)); });
    }
    else {
        var lastSizeIndex = void 0;
        for (lastSizeIndex = sizes.length - 1; lastSizeIndex >= 0; lastSizeIndex--) {
            if (sizes[lastSizeIndex] !== 0) {
                break;
            }
        }
        if (lastSizeIndex === -1) {
            //all zeros - return a copy
            return sizes.slice();
        }
        var head = sizes.slice(0, lastSizeIndex).map(function (size) { return Number(size.toFixed(precision)); });
        var tail = sizes.slice(lastSizeIndex + 1);
        var inserted = Number((sum - getSum(head)).toFixed(precision));
        return head.concat(inserted, tail);
    }
}
exports.fixFloatSizes = fixFloatSizes;
function getPlaceholderPlacement(x1, y1, x2, y2, x, y, minWidth, minHeight, isMaximized) {
    //check bounds
    var isOutOfBounds = x1 === x2 ||
        y1 === y2 ||
        (x1 < x2 && (x < x1 || x > x2)) ||
        (x2 < x1 && (x < x2 || x > x1)) ||
        (y1 < y2 && (y < y1 || y > y2)) ||
        (y2 < y1 && (y < y2 || y > y1));
    if (!isOutOfBounds) {
        var width = Math.abs(x2 - x1);
        var cx = Math.min(x1, x2) + width / 2;
        var height = Math.abs(y2 - y1);
        var cy = Math.min(y1, y2) + height / 2;
        var k = (y2 - y1) / (x2 - x1);
        //shift coords to (0;0)
        var sx = x - cx;
        var sy = y - cy;
        var doubleMinWidth = minWidth * 2;
        var doubleMinHeight = minHeight * 2;
        var isTop = (sx < 0 && sy <= k * sx) || (sx >= 0 && sy <= -k * sx);
        var isBottom = (sx < 0 && sy >= -k * sx) || (sx >= 0 && sy >= k * sx);
        var isLeft = sx < 0 && sy > k * sx && sy < -k * sx;
        var isRight = sx > 0 && sy > -k * sx && sy < k * sx;
        //vertical axis has higher priority
        if (isTop) {
            return {
                side: layout_constants_1.ItemSide.Top,
                fits: !isMaximized && width >= minWidth && height >= doubleMinHeight,
            };
        }
        if (isBottom) {
            return {
                side: layout_constants_1.ItemSide.Bottom,
                fits: !isMaximized && width >= minWidth && height >= doubleMinHeight,
            };
        }
        if (isLeft) {
            return {
                side: layout_constants_1.ItemSide.Left,
                fits: !isMaximized && height >= minHeight && width >= doubleMinWidth,
            };
        }
        if (isRight) {
            return {
                side: layout_constants_1.ItemSide.Right,
                fits: !isMaximized && height >= minHeight && width >= doubleMinWidth,
            };
        }
    }
    return {
        fits: false,
    };
}
exports.getPlaceholderPlacement = getPlaceholderPlacement;
function invalidateParents(items, fromId) {
    var _a;
    var parent = items[items[fromId].parent_id];
    if (parent) {
        return invalidateParents(tslib_1.__assign(tslib_1.__assign({}, items), (_a = {}, _a[parent.id] = tslib_1.__assign({}, parent), _a)), parent.id);
    }
    else {
        return items;
    }
}
exports.invalidateParents = invalidateParents;
function generateWrapperLayout(id, destination, parent, side) {
    var result = {
        id: string_1.uuid(),
        size: destination.size,
        parent_id: parent.id,
    };
    switch (side) {
        case layout_constants_1.ItemSide.Top: {
            return tslib_1.__assign(tslib_1.__assign({}, result), { type: layout_constants_1.LayoutModelItemType.Layout, orientation: layout_constants_1.LayoutOrientation.Vertical, item_ids: [id, destination.id] });
        }
        case layout_constants_1.ItemSide.Bottom: {
            return tslib_1.__assign(tslib_1.__assign({}, result), { type: layout_constants_1.LayoutModelItemType.Layout, orientation: layout_constants_1.LayoutOrientation.Vertical, item_ids: [destination.id, id] });
        }
        case layout_constants_1.ItemSide.Left: {
            return tslib_1.__assign(tslib_1.__assign({}, result), { type: layout_constants_1.LayoutModelItemType.Layout, orientation: layout_constants_1.LayoutOrientation.Horizontal, item_ids: [id, destination.id] });
        }
        case layout_constants_1.ItemSide.Right: {
            return tslib_1.__assign(tslib_1.__assign({}, result), { type: layout_constants_1.LayoutModelItemType.Layout, orientation: layout_constants_1.LayoutOrientation.Horizontal, item_ids: [destination.id, id] });
        }
    }
}
exports.generateWrapperLayout = generateWrapperLayout;
function flattenLayoutItems(items, fromId) {
    var from = items[fromId];
    switch (from.type) {
        case layout_constants_1.LayoutModelItemType.Plain: {
            //plain item - nothing to flattenLayoutItems
            return items;
        }
        case layout_constants_1.LayoutModelItemType.Layout: {
            if (!from.parent_id) {
                //root - just traverse children
                return from.item_ids.reduce(flattenLayoutItems, items);
            }
            //nested layout
            var amount = from.item_ids.length;
            switch (amount) {
                case 0: {
                    //empty nested layout - remove it
                    var parent_1 = items[from.parent_id];
                    switch (parent_1.type) {
                        case layout_constants_1.LayoutModelItemType.Layout: {
                            var copy = tslib_1.__assign({}, items);
                            delete copy[fromId];
                            copy[parent_1.id] = tslib_1.__assign(tslib_1.__assign({}, parent_1), { item_ids: parent_1.item_ids.filter(function (id) { return id !== fromId; }) });
                            return copy;
                        }
                    }
                    return items;
                }
                case 1: {
                    var parent_2 = items[from.parent_id];
                    switch (parent_2.type) {
                        case layout_constants_1.LayoutModelItemType.Layout: {
                            var copy = tslib_1.__assign({}, items);
                            delete copy[fromId];
                            copy[parent_2.id] = tslib_1.__assign(tslib_1.__assign({}, parent_2), { item_ids: parent_2.item_ids.map(function (id) {
                                    return id === fromId ? from.item_ids[0] : id;
                                }) });
                            copy[from.item_ids[0]] = tslib_1.__assign(tslib_1.__assign({}, copy[from.item_ids[0]]), { parent_id: from.parent_id });
                            return flattenLayoutItems(copy, from.item_ids[0]);
                        }
                    }
                    return items;
                }
                default: {
                    var result = from.item_ids.reduce(flattenLayoutItems, items);
                    var newFrom = result[fromId];
                    switch (newFrom.type) {
                        case layout_constants_1.LayoutModelItemType.Layout: {
                            if (newFrom.item_ids.length !== amount) {
                                return flattenLayoutItems(result, fromId);
                            }
                            else {
                                return result;
                            }
                        }
                    }
                    return result;
                }
            }
        }
    }
    return items;
}
exports.flattenLayoutItems = flattenLayoutItems;
function normalizeLayout(data) {
    var result = {
        root_id: data.id,
        items: {},
    };
    var traverse = function (item, parent_id) {
        if (item.items) {
            //nested layout
            var items = item.items, rest = tslib_1.__rest(item, ["items"]);
            result.items[item.id] = tslib_1.__assign(tslib_1.__assign({}, rest), { parent_id: parent_id, item_ids: item.items.map(function (nestedItem) { return nestedItem.id; }) });
            item.items.forEach(function (nestedItem) { return traverse(nestedItem, item.id); });
        }
        else {
            //plain item
            result.items[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { parent_id: parent_id });
        }
    };
    traverse(data);
    return setDefaults(result);
}
exports.normalizeLayout = normalizeLayout;
function denormalizeLayout(layout) {
    var construct = function (id) {
        var item = layout.items[id];
        switch (item.type) {
            case layout_constants_1.LayoutModelItemType.Layout: {
                var item_ids = item.item_ids, parent_id = item.parent_id, rest = tslib_1.__rest(item, ["item_ids", "parent_id"]);
                return tslib_1.__assign(tslib_1.__assign({}, rest), { items: item_ids.map(construct) });
            }
            case layout_constants_1.LayoutModelItemType.Plain: {
                //noinspection UnnecessaryLocalVariableJS
                var parent_id = item.parent_id, rest = tslib_1.__rest(item, ["parent_id"]);
                return rest;
            }
        }
    };
    return construct(layout.root_id);
}
exports.denormalizeLayout = denormalizeLayout;
function getDefaultChildrenSizes(layout, item, containerSize, precision) {
    var amount = item.item_ids.length;
    if (amount === 0) {
        return [];
    }
    if (amount === 1) {
        return [Number(containerSize.toFixed(precision))];
    }
    var items = item.item_ids.map(function (id) { return layout.items[id]; });
    var sizes = items.map(function (item) { return item.size; });
    //collect all defined sizes
    var defined = sizes.reduce(function (acc, size, i) {
        if (typeof size !== 'undefined') {
            acc[i] = size;
        }
        return acc;
    }, {});
    //get amount of undefined sizes that should stretch
    var undefinedAmount = amount - Object.keys(defined).length;
    //get average stretching size
    var averageUndefinedSize = containerSize / undefinedAmount;
    //fill undefined sizes with average values
    var filled = sizes.reduce(function (acc, size, i) {
        if (typeof defined[i] !== 'undefined') {
            acc.push(defined[i]);
        }
        else {
            acc.push(Number(averageUndefinedSize.toFixed(precision)));
        }
        return acc;
    }, []);
    return fixFloatSizes(filled, containerSize, precision);
}
exports.getDefaultChildrenSizes = getDefaultChildrenSizes;
function setDefaults(layout) {
    var items = tslib_1.__assign({}, layout.items);
    if (items[layout.root_id].size !== 1) {
        //root size is always 1
        items[layout.root_id] = tslib_1.__assign(tslib_1.__assign({}, items[layout.root_id]), { size: 1 });
    }
    var precision = 5;
    var size = 1;
    Object.keys(items).forEach(function (id) {
        var item = items[id];
        //set default type
        if (typeof item.type === 'undefined') {
            if (typeof item['item_ids'] !== 'undefined') {
                items[item.id] = tslib_1.__assign(tslib_1.__assign({}, items[item.id]), { type: layout_constants_1.LayoutModelItemType.Layout });
            }
            else {
                items[item.id] = tslib_1.__assign(tslib_1.__assign({}, items[item.id]), { type: layout_constants_1.LayoutModelItemType.Plain });
            }
        }
        //todo: find a way to mutate readonly object
        item = items[id];
        switch (item.type) {
            case layout_constants_1.LayoutModelItemType.Layout: {
                //nested layout
                //set default orientation
                if (!layout_constants_1.LayoutOrientation[item.orientation]) {
                    items[item.id] = tslib_1.__assign(tslib_1.__assign({}, item), { orientation: layout_constants_1.LayoutOrientation.Horizontal });
                }
                //set default sizes
                var sizes_1 = getDefaultChildrenSizes(layout, item, size, precision);
                item.item_ids.forEach(function (nestedItemId, i) {
                    var nestedItem = layout.items[nestedItemId];
                    if (nestedItem.size !== sizes_1[i]) {
                        items[nestedItem.id] = tslib_1.__assign(tslib_1.__assign({}, items[nestedItem.id]), { size: sizes_1[i] });
                    }
                });
            }
        }
    });
    return tslib_1.__assign(tslib_1.__assign({}, layout), { items: items });
}
exports.setDefaults = setDefaults;
function prepareLayout(layout) {
    var normalized = normalizeLayout(layout);
    var items = flattenLayoutItems(normalized.items, normalized.root_id);
    return setDefaults(tslib_1.__assign(tslib_1.__assign({}, normalized), { items: items }));
}
exports.prepareLayout = prepareLayout;
function getResizeHandlerSide(sizes, index) {
    var amount = sizes.length;
    if (amount > 1) {
        //ignore zeros
        if (sizes[index] !== 0) {
            if (index === 0) {
                //first
                //lookup to the right
                var allRightAreZeros = sizes.slice(index + 1).every(function (item) { return item === 0; });
                if (!allRightAreZeros) {
                    return layout_constants_1.ResizeHandlerSide.After;
                }
            }
            else if (index === amount - 1) {
                //last
                //lookup to the left
                var allLeftAreZeros = sizes.slice(0, index).every(function (item) { return item === 0; });
                if (!allLeftAreZeros) {
                    return layout_constants_1.ResizeHandlerSide.Before;
                }
            }
            else {
                //in the middle
                var allLeftAreZeros = sizes.slice(0, index).every(function (item) { return item === 0; });
                var allRightAreZeros = sizes.slice(index + 1).every(function (item) { return item === 0; });
                if (allLeftAreZeros) {
                    //only after or none
                    if (allRightAreZeros) {
                        //none
                    }
                    else {
                        //after
                        return layout_constants_1.ResizeHandlerSide.After;
                    }
                }
                else {
                    //before or both
                    if (allRightAreZeros) {
                        //before
                        return layout_constants_1.ResizeHandlerSide.Before;
                    }
                    else {
                        //both
                        return layout_constants_1.ResizeHandlerSide.Both;
                    }
                }
            }
        }
    }
    return undefined;
}
exports.getResizeHandlerSide = getResizeHandlerSide;
function setTransferData(event, data) {
    var dataTransfer = event.dataTransfer;
    if (!dataTransfer) {
        return;
    }
    if (data) {
        var value = void 0;
        try {
            value = tslib_1.__assign(tslib_1.__assign({}, JSON.parse(dataTransfer.getData('text'))), data);
        }
        catch (e) {
            value = data;
        }
        dataTransfer.setData('text', JSON.stringify(value));
    }
}
exports.setTransferData = setTransferData;
function setDragData(event, data) {
    var _a;
    setTransferData(event, (_a = {},
        _a[layout_constants_1.NATIVE_DRAG_DATA_KEY] = data,
        _a));
}
exports.setDragData = setDragData;
function countChildren(layout, item, orientation) {
    if (item.type === layout_constants_1.LayoutModelItemType.Layout) {
        var nestedLayouts = item.item_ids.reduce(function (acc, id) {
            var item = layout.items[id];
            if (item.type === layout_constants_1.LayoutModelItemType.Layout) {
                acc.push(item);
            }
            return acc;
        }, []);
        var nestedResult = nestedLayouts.reduce(function (acc, nestedLayout) {
            return acc + countChildren(layout, nestedLayout, orientation);
        }, 0);
        if (item.orientation === orientation) {
            return item.item_ids.length - nestedLayouts.length + nestedResult;
        }
        else {
            return nestedResult || 1; //if nestedResult is 0 there are no nested layouts - count self
        }
    }
    return 1;
}
exports.countChildren = countChildren;
function chooseByOrientation(horizontal, vertical, orientation) {
    switch (orientation) {
        case layout_constants_1.LayoutOrientation.Horizontal: {
            return horizontal;
        }
        case layout_constants_1.LayoutOrientation.Vertical: {
            return vertical;
        }
    }
}
exports.chooseByOrientation = chooseByOrientation;
function getSizeMap(layout, item, defaultMinWidth, defaultMinHeight, width, height, resizeHandlerSize, precision, ignoreId, acc) {
    if (ignoreId === void 0) { ignoreId = ''; }
    if (acc === void 0) { acc = {}; }
    var updateSizeMap = function (item, width, height) {
        var minWidth = getMinWidth(item, defaultMinWidth);
        var minHeight = getMinHeight(item, defaultMinHeight);
        if (item.type !== layout_constants_1.LayoutModelItemType.Layout || item.item_ids.length === 0) {
            //either item is plain or empty (then just take passed sizes)
            return (acc[item.id] = {
                width: width === 0 ? 0 : Math.max(width, minWidth),
                height: height === 0 ? 0 : Math.max(height, minHeight),
            });
        }
        else {
            var sizes = item.item_ids.map(function (id) {
                var item = layout.items[id];
                return item.id === ignoreId ? 0 : item.size;
            });
            var resizeHandlersDelta = (sizes.length - 1 - (ignoreId ? 1 : 0)) * resizeHandlerSize;
            switch (item.orientation) {
                case layout_constants_1.LayoutOrientation.Vertical: {
                    var finalWidth_1 = Math.max(width, minWidth);
                    var finalHeight_1 = 0;
                    var normalized = normalizeSizes(sizes, height - resizeHandlersDelta, precision);
                    var childSizes = normalized.map(function (size, i) {
                        var child = layout.items[item.item_ids[i]];
                        //pass normalized height
                        var childSize = updateSizeMap(child, width, size);
                        //collect max width
                        finalWidth_1 = Math.max(finalWidth_1, childSize.width);
                        //increase total height
                        finalHeight_1 += childSize.height;
                        return childSize;
                    });
                    //add resize handlers
                    finalHeight_1 += resizeHandlersDelta;
                    //check if finalHeight is greater then passed height
                    //and try to decrease siblings if there are some that can be shrinked
                    if (finalHeight_1 > height) {
                        var diff = finalHeight_1 - height;
                        var amountOfMinimums = childSizes.filter(function (size) { return size.height === minHeight; }).length;
                        var amountOfShrinkable = childSizes.length - amountOfMinimums;
                        if (amountOfShrinkable > 0) {
                            //still have some items to shrink
                            var shrinkDelta_1 = Number((diff / amountOfShrinkable).toFixed(precision));
                            //iterate over calculated childSizes to shrink them and adjust finalHeight
                            childSizes.forEach(function (childSize, i) {
                                if (childSize.height !== minHeight) {
                                    //shrink only items that can be shrinked
                                    var child = layout.items[item.item_ids[i]];
                                    var newSize = updateSizeMap(child, childSize.width, childSize.height - shrinkDelta_1);
                                    finalHeight_1 -= childSize.height;
                                    finalHeight_1 += newSize.height;
                                    finalHeight_1 = Number(finalHeight_1.toFixed(precision));
                                }
                            });
                        }
                        //nothing we can do here - take finalHeight which if greater than passed height
                    }
                    //store new sizes
                    acc[item.id] = {
                        width: finalWidth_1,
                        height: finalHeight_1,
                    };
                    //check if siblings should be resized
                    if (finalWidth_1 > width) {
                        //increase width of all siblings
                        normalized.forEach(function (size, i) {
                            var child = layout.items[item.item_ids[i]];
                            //finalWidth is maxWidth
                            //do not update self
                            if (acc[child.id].width !== finalWidth_1) {
                                acc[child.id] = updateSizeMap(child, finalWidth_1, acc[child.id].height);
                            }
                        });
                    }
                    return acc[item.id];
                }
                case layout_constants_1.LayoutOrientation.Horizontal: //fallthrough
                default: {
                    var finalHeight_2 = Math.max(height, minHeight);
                    var finalWidth_2 = 0;
                    var normalized = normalizeSizes(sizes, width - resizeHandlersDelta, precision);
                    var childSizes = normalized.map(function (size, i) {
                        var child = layout.items[item.item_ids[i]];
                        //pass normalized width
                        var childSize = updateSizeMap(child, size, height);
                        //collect max height
                        finalHeight_2 = Math.max(finalHeight_2, childSize.height);
                        //increase total width
                        finalWidth_2 += childSize.width;
                        return childSize;
                    });
                    //add resize handlers
                    finalWidth_2 += resizeHandlersDelta;
                    //check if finalHeight is greater then passed height
                    //and try to decrease siblings if there are some that can be shrinked
                    if (finalWidth_2 > width) {
                        var diff = finalWidth_2 - width;
                        var amountOfMinimums = childSizes.filter(function (size) { return size.width === minWidth; }).length;
                        var amountOfShrinkable = childSizes.length - amountOfMinimums;
                        if (amountOfShrinkable > 0) {
                            //still have some items to shrink
                            var shrinkDelta_2 = Number((diff / amountOfShrinkable).toFixed(precision));
                            //iterate over calculated childSizes to shrink them and adjust finalHeight
                            childSizes.forEach(function (childSize, i) {
                                if (childSize.width !== minWidth) {
                                    //shrink only items that can be shrinked
                                    var child = layout.items[item.item_ids[i]];
                                    var newSize = updateSizeMap(child, childSize.width - shrinkDelta_2, childSize.height);
                                    finalWidth_2 -= childSize.width; //
                                    finalWidth_2 += newSize.width;
                                    finalWidth_2 = Number(finalWidth_2.toFixed(precision));
                                }
                            });
                        }
                        //nothing we can do here - take finalHeight which if greater than passed height
                    }
                    //store new sizes
                    acc[item.id] = {
                        width: finalWidth_2,
                        height: finalHeight_2,
                    };
                    //check if siblings should be resized
                    if (finalHeight_2 > height) {
                        //increase height of all siblings
                        normalized.forEach(function (size, i) {
                            var child = layout.items[item.item_ids[i]];
                            //finalHeight is maxHeight
                            //do not update self
                            if (acc[child.id].height !== finalHeight_2) {
                                acc[child.id] = updateSizeMap(child, acc[child.id].width, finalHeight_2);
                            }
                        });
                    }
                    return acc[item.id];
                }
            }
        }
    };
    updateSizeMap(item, width, height);
    return acc;
}
exports.getSizeMap = getSizeMap;
function normalizeSizes(sizes, containerSize, precision) {
    var amount = sizes.length;
    if (amount === 0) {
        return [];
    }
    if (amount === 1) {
        return [Number(containerSize.toFixed(precision))];
    }
    var k = getSum(sizes) / containerSize;
    var normalized = sizes.map(function (size) { return size / k; });
    return fixFloatSizes(normalized, containerSize, precision);
}
exports.normalizeSizes = normalizeSizes;
function isEmptyRoot(layout, item) {
    return item.id === layout.root_id && item.item_ids.length === 0;
}
exports.isEmptyRoot = isEmptyRoot;
function getMinWidth(item, defaultMinWidth) {
    return item.type === layout_constants_1.LayoutModelItemType.Plain && item.props !== undefined && item.props.minWidth !== undefined
        ? item.props.minWidth
        : defaultMinWidth;
}
exports.getMinWidth = getMinWidth;
function getMinHeight(item, defaultMinHeight) {
    return item.type === layout_constants_1.LayoutModelItemType.Plain && item.props !== undefined && item.props.minHeight !== undefined
        ? item.props.minHeight
        : defaultMinHeight;
}
exports.getMinHeight = getMinHeight;
//# sourceMappingURL=layout.utils.js.map